# -*- coding: utf-8 -*-
import unittest2 as unittest

from zope.site.hooks import setSite

from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles

from collective.caching.hostname.testing import INTEGRATION_TESTING

PROJECTNAME = 'collective.caching.hostname'


class BaseTestCase(unittest.TestCase):
    """base test case to be used by other tests"""

    layer = INTEGRATION_TESTING

    def setUpUser(self):
        setRoles(self.portal, TEST_USER_ID, ['Manager',
                                             'Editor',
                                             'Reviewer', ])
        login(self.portal, TEST_USER_NAME)

    def setUp(self):
        portal = self.layer['portal']
        setSite(portal)
        self.portal = portal
        self.qi = getattr(self.portal, 'portal_quickinstaller')
        self.st = getattr(self.portal, 'portal_setup')
        self.setUpUser()


class TestInstall(BaseTestCase):
    """ensure product is properly installed"""

    def test_installed(self):
        self.assertTrue(self.qi.isProductInstalled(PROJECTNAME),
                        '%s not installed' % PROJECTNAME)

    def test_installed_plone_app_caching(self):
        dependency = 'plone.app.caching'
        self.assertTrue(self.qi.isProductInstalled(dependency),
                        '%s not installed' % dependency)


class TestUninstall(BaseTestCase):
    """ensure product is properly uninstalled"""

    def setUp(self):
        BaseTestCase.setUp(self)
        self.qi.uninstallProducts(products=[PROJECTNAME])

    def test_uninstalled(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
import logging

from zope import component

from Products.CMFCore.utils import getToolByName
from Products.GenericSetup import interfaces as gsinterfaces
from Products.GenericSetup.upgrade import listUpgradeSteps

PROJECT = 'collective.caching.hostname'


def fromZero(context):
    ''' Upgrade from Zero to version 1000
    '''
    # Do something here
    pass


# -*- coding:utf-8 -*-


def getHostname(request):
    ''' Get the hostname of the request '''
    hostname = ''
    # Using VHM
    if 'VIRTUAL_URL_PARTS' in request:
        parts = request.get('VIRTUAL_URL_PARTS', [])
        hostname = parts and parts[0] or ''

    if not hostname:
        hostname = request.get('SERVER_URL', '')
    return hostname



# -*- coding: utf-8 -*-
import logging

from Products.CMFCore.utils import getToolByName

from Products.GenericSetup.upgrade import listUpgradeSteps


_PROJECT = 'collective.caching.hostname'
_PROFILE_ID = 'collective.caching.hostname:default'


def run_upgrades(context):
    ''' Run Upgrade steps
    '''
    if context.readDataFile('collective.caching.hostname-default.txt') is None:
        return
    logger = logging.getLogger(_PROJECT)
    site = context.getSite()
    setup_tool = getToolByName(site, 'portal_setup')
    version = setup_tool.getLastVersionForProfile(_PROFILE_ID)
    upgradeSteps = listUpgradeSteps(setup_tool, _PROFILE_ID, version)
    sorted(upgradeSteps, key=lambda step: step['sortkey'])

    for step in upgradeSteps:
        oStep = step.get('step')
        if oStep is not None:
            oStep.doStep(setup_tool)
            msg = "Ran upgrade step %s for profile %s" % (oStep.title,
                                                          _PROFILE_ID)
            setup_tool.setLastVersionForProfile(_PROFILE_ID, oStep.dest)
            logger.info(msg)



# -*- coding: utf-8 -*-

from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import collective.caching.hostname
        self.loadZCML(package=collective.caching.hostname)

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        self.applyProfile(portal, 'collective.caching.hostname:default')
        wt = portal['portal_workflow']
        wt.setDefaultChain('simple_publication_workflow')



FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name='collective.caching.hostname:Integration',
    )
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name='collective.caching.hostname:Functional',
    )



# -*- coding:utf-8 -*-

from zope.interface import implements
from zope.interface import classProvides
from zope.interface import Interface
from zope.component import adapts

from zope.publisher.interfaces.http import IHTTPRequest

from plone.caching.interfaces import ICachingOperation
from plone.caching.interfaces import ICachingOperationType
from plone.caching.utils import lookupOptions

from plone.app.caching.operations.utils import doNotCache
from plone.app.caching.operations.utils import getContext
from plone.app.caching.operations.default import BaseCaching

from collective.caching.hostname.operations.utils import getHostname

from plone.app.caching.interfaces import _

try:
    from Products.ResourceRegistries.interfaces import ICookedFile
    from Products.ResourceRegistries.interfaces import IResourceRegistry
    HAVE_RESOURCE_REGISTRIES = True
except ImportError:
    HAVE_RESOURCE_REGISTRIES = False


class BaseHostnameCaching(BaseCaching):
    """A generic caching operation class based on BaseCaching from
    plone.app.caching which adds a **hostnames_nocache** option
    """
    implements(ICachingOperation)
    adapts(Interface, IHTTPRequest)

    # Type metadata
    classProvides(ICachingOperationType)

    title = _(u"Generic caching w/ hostname support")
    description = _(u"Through this operation, all standard caching functions "
                    u"can be performed via various combinations of the optional "
                    u"parameter settings. For most cases, it's probably easier "
                    u"to use one of the other simpler operations (Strong caching, "
                    u"Moderate caching, Weak caching, or No caching).")
    prefix = 'collective.caching.hostname.baseCaching'
    options = ('maxage', 'smaxage', 'etags', 'lastModified', 'ramCache',
               'vary', 'anonOnly', 'hostnames_nocache')

    hostnames_nocache = []

    def interceptResponse(self, rulename, response, class_=None):
        options = lookupOptions(class_ or self.__class__, rulename)
        hostname = getHostname(self.request)
        if hostname in options.get('hostnames_nocache',
                                   self.hostnames_nocache):
            return None
        return super(BaseHostnameCaching, self).interceptResponse(rulename,
                                                                  response,
                                                                  class_=None)

    def modifyResponse(self, rulename, response, class_=None):
        options = lookupOptions(class_ or self.__class__, rulename)
        hostname = getHostname(self.request)
        if hostname in options.get('hostnames_nocache',
                                   self.hostnames_nocache):
            doNotCache(self.published, self.request, response)
        else:
            return super(BaseHostnameCaching, self).modifyResponse(rulename,
                                                                   response,
                                                                   class_=None)


class WeakCaching(BaseHostnameCaching):
    """Weak caching operation. A subclass of the generic BaseCaching
    operation to help make the UI approachable by mortals
    """

    # Type metadata
    classProvides(ICachingOperationType)

    title = _(u"Weak caching w/ hostname support")
    description = _(u"Cache in browser but expire immediately and enable 304 "
                    u"responses on subsequent requests. 304's require configuration "
                    u"of the 'Last-modified' and/or 'ETags' settings. If Last-Modified "
                    u"header is insufficient to ensure freshness, turn on ETag "
                    u"checking by listing each ETag components that should be used to "
                    u"to construct the ETag header. "
                    u"To also cache public responses in Zope memory, set 'RAM cache' to True. ")
    prefix = 'collective.caching.hostname.weakCaching'
    sort = 3

    # Configurable options
    options = ('etags', 'lastModified', 'ramCache', 'vary',
               'anonOnly', 'hostnames_nocache')

    # Default option values
    maxage = 0
    smaxage = etags = vary = None
    lastModified = ramCache = anonOnly = False
    hostnames_nocache = []


class ModerateCaching(BaseHostnameCaching):
    """Moderate caching operation. A subclass of the generic BaseCaching
    operation to help make the UI approachable by mortals
    """

    # Type metadata
    classProvides(ICachingOperationType)

    title = _(u"Moderate caching w/ hostname support")
    description = _(u"Cache in browser but expire immediately (same as 'weak caching'), "
                    u"and cache in proxy (default: 24 hrs). "
                    u"Use a purgable caching reverse proxy for best results. "
                    u"Caution: If proxy cannot be purged, or cannot be configured "
                    u"to remove the 's-maxage' token from the response, then stale "
                    u"responses might be seen until the cached entry expires. ")
    prefix = 'collective.caching.hostname.moderateCaching'
    sort = 2

    # Configurable options
    options = ('smaxage', 'etags', 'lastModified', 'ramCache', 'vary',
               'anonOnly', 'hostnames_nocache')

    # Default option values
    maxage = 0
    smaxage = 86400
    etags = vary = None
    lastModified = ramCache = anonOnly = False
    hostnames_nocache = []


class StrongCaching(BaseHostnameCaching):
    """Strong caching operation. A subclass of the generic BaseCaching
    operation to help make the UI approachable by mortals
    """

    # Type metadata
    classProvides(ICachingOperationType)

    title = _(u"Strong caching w/ hostname support")
    description = _(u"Cache in browser and proxy (default: 24 hrs). "
                    u"Caution: Only use for stable resources "
                    u"that never change without changing their URL, or resources "
                    u"for which temporary staleness is not critical.")
    prefix = 'collective.caching.hostname.strongCaching'
    sort = 1

    # Configurable options
    options = ('maxage', 'smaxage', 'etags', 'lastModified', 'ramCache',
               'vary', 'anonOnly', 'hostnames_nocache')

    # Default option values
    maxage = 86400
    smaxage = etags = vary = None
    lastModified = ramCache = anonOnly = False
    hostnames_nocache = []

if HAVE_RESOURCE_REGISTRIES:

    class ResourceRegistriesCaching(StrongCaching):
        """Override for StrongCaching which checks ResourceRegistries
        cacheability
        """

        adapts(ICookedFile, IHTTPRequest)

        def interceptResponse(self, rulename, response):
            return super(ResourceRegistriesCaching, self).interceptResponse(rulename, response, class_=StrongCaching)

        def modifyResponse(self, rulename, response):
            registry = getContext(self.published, IResourceRegistry)

            if registry is not None:
                if registry.getDebugMode() or not registry.isCacheable(self.published.__name__):
                    doNotCache(self.published, self.request, response)
                    return

            super(ResourceRegistriesCaching, self).modifyResponse(rulename, response, class_=StrongCaching)



# -*- coding:utf-8 -*-
import os
from setuptools import setup, find_packages

version = '0.5'

setup(name='collective.caching.hostname',
      version=version,
      description="Hostname based blacklist caching operations for Plone",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 4.1",
        "Framework :: Plone :: 4.2",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='python plone zope simples_consultoria',
      author='Simples Consultoria',
      author_email='products@simplesconsultoria.com.br',
      url='http://www.simplesconsultoria.com.br',
      license='gpl',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective',
                         'collective.caching'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'five.grok',
          'plone.app.caching',
      ],
      extras_require={
        'test': ['plone.app.testing'],
        },
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
